@@ -11,8 +11,8 @@ exec $THESH "$0" "$@"
 #  Boust:
 #    a tcl/tk text reader that formats the text boustrophedon.
 #
-#  Version 0.14
-#  Last modified 2001.11.12
+#  Version 0.161
+#  Last modified 2002.04.21
 #
 #
 #  AUTHOR: Simmon Keith Barney <boust@traevoli.com>
@@ -32,7 +32,7 @@ exec $THESH "$0" "$@"
 #  the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 #  USA.
 
-set Version "0.14"
+set Version "0.161"
 set contents ""
 set ft ""
 set lines ""
@@ -46,13 +46,14 @@ set psline 1
 set direction 1
 set left $maxX
 set iscolor 0
+set rongo 0
 set print 0
 set printFile boust_out.ps
 set prnOut ""
-set pagew 612
-set pageh 792
-set lrmargin 60
-set tbmargin 60
+set pagew 8.5
+set pageh 11
+set lrmargin 1
+set tbmargin 1
 #set psfontsize [expr [expr [expr $pagew - $lrmargin - $lrmargin]/ $maxX] * 1.666666667]
 #set psfontsize 14
 set Pages 1
@@ -60,27 +61,41 @@ set check ""
 set lastcheck ""
 set printOnly 0
 set userfontsize "unspecified"
+set spacechar " "
+set pagebreak  "<boust-pgbk>"
+set linebreak  "<boust-lnbk>"
+set turnaround "<boust-revlnbk>"
+set space       "<boust-sp>"
 
 set fwfont 10x20-iso8859-forwards
 set bwfont 10x20-iso8859-backwards
-set fwc #c0ffd0
-set bwc #ffffd0
+set udfont 10x20-iso8859-upsidedown
+set fwc grey
+set bwc darkgrey
+
+set pntype "none"
 
 set printhelp "
 boust \[--h --o --cpl --pw --ph --lr --tm --f\] \[input_file\]
                                             (defaults to README)
   --h     this help screen
+  --r	  specifies Rongorongo (font rotated instead of mirrored)
 
   --o \[filename\] ps output file (or - for stdout)
    	             inhibits loading of tk interface
   --cpl # 	    characters per line (default 60)
-  --pw  # 	    page width* 	(default 612)
-  --ph  # 	    page height*        (default 792)
-  --lr  # 	    left-right margin*  (default 60)
-  --tb  # 	    top-bottom margin*  (default 60)
+  --pw  # 	    page width* 	(default 8.5)
+  --ph  # 	    page height*        (default 11)
+  --lr  # 	    left-right margin*  (default 1)
+  --tb  # 	    top-bottom margin*  (default 1)
   --f   # 	    font size 	        (precalculated)
+  --pn  none	    no page numbering on .ps output
+        up	    @ top of page
+	down	    @ bottom of page
+	both	    @ top & bottom
+	alt	    alternate top/bottom each page
 
-     * - in 72nds of an inch
+     * - in inches
 "
 
 if {[llength $argv] > 0} {
@@ -94,13 +109,12 @@ if {[llength $argv] > 0} {
      }
    }
 
-   #Every parametere (except --h) should have a value.  So we can assume
-   #that if there's an odd number, there's a file specified
-   while {[llength $para] > 1} {
+   while {[llength $para] > 0} {
    	 #puts stderr "[lindex $para 0] [lindex $para 1]"
 	 
 	 set parameter [lindex $para 0]
 	 set value [lindex $para 1]
+	 set filepar ""
 	 
 	 switch -exact -- $parameter {
 	   --o { set printFile $value; set printOnly 1; }
@@ -109,14 +123,26 @@ if {[llength $argv] > 0} {
 	   --ph {set pageh $value;}
 	   --lr {set lrmargin $value;}
 	   --tb {set tbmargin $value;}
-	   --f { set userfontsize $value; }
+	   --f  {set userfontsize $value; }
+	   --pn { if {[regexp "none|top|bottom|double|alt" $value]} { 
+	   	      set pntype $value
+		  } else {
+		    puts stderr "\"$value\" not a valid page numbering type\n"
+		  }
+	       ;}
+	   --r { set rongo 1;}
+	   default { set filepar $parameter; }
 	 }
 	 
-	 #Chop off the first two parameters
-  	 set para [lrange $para 2 [expr [llength $para]-1]]
+	 #Chop off used parameter(s)
+	 if {$parameter == "--r"} {
+	   set para [lrange $para 1 [expr [llength $para]-1]]
+	 } else {
+  	   set para [lrange $para 2 [expr [llength $para]-1]]
+	 }
    }
-   if {[llength $para] == 1} {
-       set fileName [lindex $para 0]  
+   if {$filepar != ""} {
+       set fileName $filepar
    } else {
      puts stderr "No file specified.  Defaulting to 'README'.  "
      set fileName README
@@ -143,6 +169,7 @@ scrollbar .yscrollbar -command ".read yview"
 pack .yscrollbar -side right -fill y -in .top -expand false
 .read tag configure fw -font $fwfont
 .read tag configure bw -font $bwfont
+.read tag configure ud -font $udfont
 .read tag configure fwcolor -background $fwc
 .read tag configure bwcolor -background $bwc
 pack .read -side right -fill both -expand true -in .top
@@ -158,22 +185,31 @@ button .reload -text Reload -command {
   .read delete 0.0 end
   main
 }
+checkbutton .rongo -text "Rongorongo" -variable rongo -command {
+	.reformat invoke}
 checkbutton .coloron -text "On/Off" -variable iscolor -command {
 	.reformat invoke}
 button .print -text "Print" -command {
 	print_options
 }
+bind . <p> {print_options}
+bind . <P> {print_options}
 entry .fwcolor_entry -textvariable fwc
 entry .bwcolor_entry -textvariable bwc
-label .fwcolor_label -text Color:
-label .bwcolor_label -text Color:
+label .fwcolor_label -text &
+label .bwcolor_label -text Colors:
+label .spacer1 -text "       "
+label .spacer2 -text "                                "
+pack .coloron -in .aboveb -side right
 pack .fwcolor_entry -in .aboveb -side right
 pack .fwcolor_label -in .aboveb -side right
 pack .bwcolor_entry -in .aboveb -side right
 pack .bwcolor_label -in .aboveb -side right
-pack .coloron -in .aboveb -side right
+pack .spacer1 -in .aboveb -side right
+pack .rongo -in .aboveb -side right
 pack .print -in .aboveb -side left
 pack .quit -in .bottom -side right
+pack .spacer2 -in .bottom -side right
 pack .reformat -in .bottom -side right 
 pack .reload -in .bottom -side right
 label .status 
@@ -186,8 +222,9 @@ pack .entry_label -side right -in .bottom
 focus .yscrollbar
 
 bind . <Prior> {.read yview scroll -1 page}
-bind . <Next> {.read yview scroll 1 page}
 bind . <Up> {.read yview scroll -1 units}
+bind . <Next> {.read yview scroll 1 page}
+bind . <space> {.read yview scroll 1 page}
 bind . <Down> {.read yview scroll 1 units}
 bind .read <FocusIn> {focus .}
 bind .filename_entry <Return> {.reload invoke}
@@ -202,6 +239,8 @@ bind .bwcolor_entry <Return> {
 	if {$iscolor} {.reformat invoke}
      }
 bind . <Escape> {quit_dialog}
+bind . <q> {quit_dialog}
+bind . <Q> {quit_dialog}
 bind .read <Configure> {reset_geometry}
 
 proc quit_dialog {} {
@@ -215,7 +254,13 @@ proc quit_dialog {} {
   pack .quit_dialog.yes -side left
   pack .quit_dialog.no -side right
   bind .quit_dialog <Escape> {destroy .quit_dialog}
+  bind .quit_dialog <n> {destroy .quit_dialog}
+  bind .quit_dialog <N> {destroy .quit_dialog}
   bind .quit_dialog <Return> {exit}
+  bind .quit_dialog <q> {exit}
+  bind .quit_dialog <y> {exit}
+  bind .quit_dialog <Q> {exit}
+  bind .quit_dialog <Y> {exit}
 
   #puts stderr [winfo geometry .quit_dialog]
  
@@ -242,6 +287,7 @@ global lrmargin
 global tbmargin
 global Pages
 global maxX
+global pntype
 
   toplevel .print_dialog
   wm title .print_dialog {Printing Options}
@@ -267,22 +313,49 @@ global maxX
   entry .print_dialog.tbmargin -textvariable tbmargin
   label .print_dialog.pagew_label -text "Page Width:"
   label .print_dialog.pageh_label -text "Page Height:"
-  label .print_dialog.size_label -text "Font Size:"
+  label .print_dialog.size_label -text "Font Size (calculated):"
   label .print_dialog.lrmargin_label -text "Left-Right Margin:"
   label .print_dialog.tbmargin_label -text "Top-Bottom Margin:"
-  pack .print_dialog.pagew_label
-  pack .print_dialog.pagew
-  pack .print_dialog.pageh_label
-  pack .print_dialog.pageh
-  pack .print_dialog.lrmargin_label
-  pack .print_dialog.lrmargin
-  pack .print_dialog.tbmargin_label
-  pack .print_dialog.tbmargin
+  radiobutton .print_dialog.pntop -text "Top" -variable pntype -value "top"
+  radiobutton .print_dialog.pnbot -text "Bottom" -variable pntype -value "bottom"
+  radiobutton .print_dialog.pntb -text "Double" -variable pntype -value "double"
+  radiobutton .print_dialog.pnalt -text "Alternated" -variable pntype -value "alt"
+  radiobutton .print_dialog.pnno  -text "None" -variable pntype -value "none"
+  label .print_dialog.pagenum -text "Page Number Location"
+  pack .print_dialog.pagew_label 
+  pack .print_dialog.pagew 
+  pack .print_dialog.pageh_label 
+  pack .print_dialog.pageh 
+  pack .print_dialog.lrmargin_label 
+  pack .print_dialog.lrmargin 
+  pack .print_dialog.tbmargin_label 
+  pack .print_dialog.tbmargin 
   pack .print_dialog.size_label
   pack .print_dialog.size
+  pack .print_dialog.pagenum -side top 
+  pack .print_dialog.pntop -side top -side left
+  pack .print_dialog.pnbot -side top -side left
+  pack .print_dialog.pntb -side top -side left
+  pack .print_dialog.pnalt -side top -side left
+  pack .print_dialog.pnno -side top -side left
 
   bind .print_dialog.print_file <Return> {.print_dialog.print invoke}
   bind .print_dialog <Escape> {.print_dialog.cancel invoke}
+  bind .print_dialog <q> {.print_dialog.cancel invoke}
+  bind .print_dialog <Q> {.print_dialog.cancel invoke}
+  
+  set mousex [winfo pointerx .] 
+  set mousey [winfo pointery .] 
+  #set offsetx [expr [winfo width .quit_dialog] /2]
+  #set offsety [expr [winfo height .quit_dialog] /2]
+  #I can't get the geometry of the window I'm creating!
+  #Static values will have to do for now.
+  set offsetx 25
+  set offsety 275
+  set printx [expr $mousex - $offsetx]
+  set printy [expr $mousey - $offsety]
+
+  wm geometry .print_dialog +$printx+$printy
 }
 
 
@@ -314,10 +387,15 @@ global argv
 global maxX
 global maxY
 global left
+global rongo
 if {$printOnly == 0} {
   global .read
   global .yscrollbar
 }
+global pagebreak
+global linebreak
+global turnaround
+global space
 global fileName
 
 global contents; set contents {}
@@ -352,16 +430,80 @@ if {$file_error} {
 
         #puts "preformat begin"
 #       Begin Preformat section
+###
+### What we want to do here is get each paragraph onto its own line ###
 
-# Remove all \r (carriage return)
-regsub -all "\r" $contents "" contents
+# Remove all \r (carriage return) and \v (vertical tab)
+regsub -all "\r" $filecontents "" contents
+regsub -all "\v" $filecontents "" contents
 
-# Remove all \n that don't make a paragraph
-regsub -all "\n(\[^(\n\t*|\n(    +)*|\t|   +)])" \
-        $filecontents { \1} contents
+  #Seperate words from adjacent mark-up
+  foreach tag [list $linebreak $pagebreak $space $turnaround] {
+    regsub -nocase -all "$tag" $contents " $tag " contents
+  }
+
+# Remove all \n from within paragraphs (see **)
+  # ** A newline that DOES have newline(s) AND/OR (tab OR three+ whitespace)
+  # ** following it, should be replaced with a \v (vertical tab)
+
+regsub -all "\n\n" $contents "\n \n" contents
+  # ** Put a space between double newlines, so when we split, we'll have
+  # ** a blank line instead of no line
+  
+set listed_contents [split $contents \n]
+  # ** Split the string into a list by newline, 
+
+set contents ""
+set lastline 0
+
+foreach line $listed_contents {
+  # ** Go thru line by line checking our criteria.
+  if {[regexp "^\[\t \]*\$" $line]} {
+    #If this line is blank (empty or just whitespace), 
+    #append a vertical tab
+      set contents "$contents\v"
+      set lastline "\n"
+  } elseif {[regexp "^\[ \t\]\[ \t\]\[ \t\]+" $line match]} {
+    #If the line starts with three whitespace
+    if {$lastline != "\n"} {
+      if {$lastline != $match} {
+        #Add a paragraph break (\v) if the last line wasn't 
+        #blank or indented the same way
+        set contents "$contents\v"
+        set lastline $match
+      }
+    }
+    #Append this line to our string
+    set contents "$contents $line"
+  } elseif {[regexp "^\t+" $line match]} {
+    #If the line starts with at least one tab
+    if {$lastline != "\n"} {
+      if {$lastline != $match} {
+        #Add a paragraph break (\v) if the last line wasn't a blank line
+	#or indented the same
+        set contents "$contents\v"
+        set lastline $match
+      }
+    }
+    #Append this line to our string
+    set contents "$contents $line"
+  } else {
+    #Append this line to our string
+    set contents "$contents $line"
+    set lastline 0
+  }
+}
+
+#Put a \v (vertical tab) at the end of the file.
+#  (I'm not sure why, but if I don't -- with the new paragraph
+#   detection routine -- it'll chop off the last line.)
+#set contents "$contents\v"
+
+#Remove all whitespace at the beginning of the file
+regsub "^\[ \t\v\n\]*(\[^ \t\v\v\])" $contents {\1} contents
 
 #Remove all whitespace at beginning of paragraph
-regsub -all "(^|\n)\[ \t]+" $contents {\1} contents
+regsub -all "(^|\v)\[ \t]+" $contents {\1} contents
 
 #Replace all remaining tabs with a space
 regsub -all "\t" $contents " " contents
@@ -369,11 +511,14 @@ regsub -all "\t" $contents " " contents
 #Replace multiple instances of spaces with single space
 regsub -all " +" $contents " " contents
 
-#Replace whitespace that occurs just before a newline
-regsub -all " \n" $contents "\n" contents
+# ** change \v back to \n
+regsub -all "\v" $contents "\n" contents
 
-#Replace all multiple returns with single return
-regsub -all "\n\n*" $contents "\n" contents
+#Replace whitespace that occurs just before a newline
+regsub -all "\[\t \]*\n" $contents "\n" contents
+
+#Replace all multiple breaks with single break
+regsub -all "\n\n+" $contents "\n" contents
 
 #       End PreFormat section
         #puts "preformat end"
@@ -407,8 +552,14 @@ global pagew
 global pageh
 global lrmargin
 global tbmargin
+global pntype
 global Pages 
 
+set lrm [expr $lrmargin * 72]
+set tbm [expr $tbmargin * 72]
+set pw [expr $pagew * 72]
+set ph [expr $pageh * 72]
+
 if {$printFile == "-"} {
   set prnOut stdout
 } else {
@@ -431,14 +582,14 @@ fconfigure $prnOut -buffering line
 
 set ps_header "%!PS-Adobe-2.0
 %%PageOrder: Ascend   
-%%BoundingBox: 0 0 $pagew $pageh
+%%BoundingBox: 0 0 $pw $ph
 %%Creator: boust.tcl Version $Version
 %define variables & procedures
 /size $psfontsize def  
-/pagew $pagew def
-/pageh $pageh def
-/lrmargin $lrmargin def
-/tbmargin $tbmargin def
+/pagew $pw def
+/pageh $ph def
+/lrmargin $lrm def
+/tbmargin $tbm def
 /lm lrmargin def
 /rm pagew lrmargin sub def
 /tm pageh tbmargin sub size sub def
@@ -454,14 +605,53 @@ size scalefont
 size scalefont
 \[ -1 0 0 1 1 0 \] makefont
         setfont} def\n
-%init
-%%Page: 1
-newpath 
-lm tm moveto
+/upsidedown {myfont findfont
+size scalefont
+\[ -1 0 0 -1 1 size 2 div \] makefont
+        setfont} def\n
+/udforward {myfont findfont
+size scalefont
+\[ 1 0 0 -1 1 size 2 div \] makefont
+        setfont} def
+
+/top-center { pagew 2 div tm tbmargin 2 div add moveto } def
+/bottom-center { pagew 2 div bm tbmargin 2 div sub moveto } def
+
+/top_pagenumber {
+  /pagenumber exch def
+  top-center
+  forward
+  (-) show
+  pagenumber 3 string cvs show
+  (\\))show
+  top-center
+  backward
+  pagenumber 3 string cvs show
+  (\\))show
+} def
+
+/bot_pagenumber {
+  /pagenumber exch def
+  bottom-center
+  forward
+  (-) show
+  pagenumber 3 string cvs show
+  (\\))show
+  bottom-center
+  backward
+  pagenumber 3 string cvs show
+  (\\))show
+} def
+
 0 0 0 setrgbcolor\n
-%put text\n"
+%put text
+%%Page: 1 1
+newpath
+\n"
 
 puts $prnOut $ps_header
+PageNumbers
+puts $prnOut "lm tm moveto"
 
 set print 1
 postformat
@@ -491,29 +681,21 @@ global x
 global maxX
 global left
 global direction
+global pntype
 
+set ph [expr $pageh * 72]
+set tbm [expr $tbmargin * 72]
+set lrm [expr $lrmargin * 72]
 set psx $x
 
-#  puts $prnOut "
-#  newpath
-#  lm tm moveto
-#  rm tm lineto
-#  rm bm lineto
-#  lm bm lineto
-#  lm tm lineto
-#  stroke
-#  "
-
-
- #puts $prnOut "save"
-  puts $prnOut "showpage"	  	  
-  set psline [expr $pageh - $tbmargin - $psfontsize ]
+  puts $prnOut "showpage"
+  set psline [expr $ph - $tbm - $psfontsize ]
   incr Pages
-  puts $prnOut "%%Page: $Pages"
+  puts $prnOut "%%Page: $Pages $Pages"
   puts $prnOut "newpath"
   
- #puts $prnOut "restore\ncurrentpoint pop tm moveto"
-
+  PageNumbers  
+  
   if {$direction == 1} {
     puts $prnOut "lm tm moveto"
   } else {
@@ -521,6 +703,37 @@ set psx $x
   }
 }
 
+proc PageNumbers {} {
+global Pages
+global prnOut
+global pntype
+
+  switch -exact -- $pntype {
+     top 	{ puts $prnOut "$Pages top_pagenumber"; }
+     bottom 	{ puts $prnOut "$Pages bot_pagenumber";}
+     double	{ puts $prnOut "$Pages top_pagenumber";
+     		  puts $prnOut "$Pages bot_pagenumber";}
+     alt	{
+     		  if {$Pages != 0} {
+		    set evenorodd [expr $Pages / 2.0]
+		    if {[regexp {\.0} $evenorodd]} {
+		                #####
+		    #Make sure this regex is in braces, not quotes.
+		    #  Otherwise it'll fuck up every 20 pages or so...
+			  
+		    #If the page number is even (If it's divisible by two, 
+		    #	a zero will follow the decimal point)
+		      puts $prnOut "$Pages top_pagenumber"
+		    } else {
+		      puts $prnOut "$Pages bot_pagenumber"
+		    }
+		  };
+     	        }
+     none	{ puts $prnOut "%% No visible pagenumber" ;} 				
+   }
+
+}
+
 proc postformat {} {
 global direction
 global contents
@@ -547,6 +760,10 @@ global lrmargin
 global tbmargin
 global Pages
 
+set ph [expr $pageh * 72]
+set tbm [expr $tbmargin * 72]
+set lrm [expr $lrmargin * 72]
+
 set Pages 1
 
   #puts "postformat"
@@ -554,7 +771,8 @@ if {$printOnly ==0} {
   .status configure -text "Formatting..."
 }
 
-set psline [expr $pageh - $tbmargin - $psfontsize ]
+set ph [expr $pageh * 72]
+set psline [expr $ph - $tbm - $psfontsize ]
 set y 1
 set x 0
 set ft $contents
@@ -574,6 +792,7 @@ set ft [split $ft "\n"]
 set par 0
 set word 0
 
+
 # scott code
 #puts "start postformat foreach"
   foreach paragraph $ft {
@@ -607,8 +826,15 @@ set output2 ""
 set outputlength 0
 set left $maxX
 set size 0
-set spacechar " "
+global spacechar
 set count2 0
+global rongo
+global pagebreak
+global linebreak
+global turnaround
+global space
+
+
 
 while {[lindex $everything $word] !=""} {
 #puts stderr "while loop"
@@ -616,6 +842,122 @@ while {[lindex $everything $word] !=""} {
 global check
 global lastcheck
 
+ while {[regexp -nocase "$linebreak|$pagebreak|$turnaround|$space" [lindex $everything $word]]} {
+
+  if {[lindex $everything $word] == $pagebreak} {
+  #Start a new page at mark-up
+  
+    if {$print} { 
+      if {$psline != [expr $ph - $tbm - $psfontsize]} {
+        #If we're NOT at the first line of the page,
+	if {![regexp "^$spacechar*\$" $output]} {
+	#If there's something (other than whitespace) on the line
+	  #puts stderr "\[Soft PageBreak\]"
+	  #puts stderr "  ($output)"
+	  set everything [linsert $everything [expr $word + 1] $linebreak]
+	  set everything [linsert $everything [expr $word + 2] $pagebreak]	
+	} else {
+	  #go to the new page
+	  #puts stderr "\[Page Break\]"
+          NewPage
+	}
+      } else {
+	if {![regexp "^$spacechar*\$" $output]} {
+	#if we ARE on the first line, but there IS text:
+	#      	   (something other than whitespace)
+	  #insert a $linebreak and a $newpage
+	  
+	  #puts stderr "\[Soft (firstline) PageBreak\]"
+	  #puts stderr "  ($output)"
+	  set everything [linsert $everything [expr $word + 1] $linebreak]
+	  set everything [linsert $everything [expr $word + 2] $pagebreak]
+	} else {
+          #puts stderr "\[Denied PageBreak\]"
+	}
+      }
+    }
+  #Skip to the next word, so the mark-up isn't displayed.
+    incr word
+  } ;#end if-pagebreak
+  
+  if {[regexp -nocase "$linebreak|$turnaround" [lindex $everything $word]]} {
+  #Start a new line at mark-up
+    
+   set nextword [lindex $everything [expr $word + 1]]
+   
+   if {[lindex $everything $word] == "$linebreak"} {
+     set remaining $left
+   } else {
+     set remaining [expr $maxX - $left]
+   }
+   if {$nextword == "$pagebreak"} {
+     #If the next word is a pagebreak, it doesn't matter if there's 
+     #room for the 'word', cause it'll break the page and then a
+     #a new word of real length will be in the queue
+     set remaining $maxX
+   }
+   #puts stderr "[string length $nextword] into $remaining"
+    
+   if {[string length $nextword] < $remaining} {
+     #puts stderr "Fits\n"
+  
+    for {set i 0} {$i < $left} {incr i} {
+      append output "$spacechar"
+    }
+ 
+    write_line $direction $output
+    set output ""
+    incr y
+    if {$print} {
+      #incr psline [expr $psfontsize * -1]
+      set psline [expr $psline + [expr $psfontsize * -1.0] ]
+      if { $psline <= $tbm } {
+  	  #NewPage
+	  set everything [linsert $everything [expr $word + 1] $pagebreak]
+      }
+    }
+
+      if {[lindex $everything $word] == $linebreak} {
+        ps_empty
+        set direction [expr $direction * -1]
+        set left [expr $maxX - $left]
+	if {[lindex $everything [expr $word -1]] == $linebreak} {
+	  #Normally, there's one trailing space after a word, but we
+	  #want to negate that for a linebreak
+	  incr left -1
+	} else {
+	  #However, if the last "word" *was* a linebreak, that trailing space
+	  #didn't occur.
+          incr left +1
+	}
+        for {set i 0} {$i < [expr $maxX - $left]} {incr i} {
+          append output "$spacechar"
+        }    
+      } else {
+        set direction [expr $direction * -1]
+        set left [expr $maxX - $left]
+        incr left -1	
+        for {set i 0} {$i < [expr $maxX - $left]} {incr i} {
+          append output "$spacechar"
+        }
+      }
+   } {
+     #puts stderr "Doesn't fit\n";
+   }
+
+  #Skip to the next word, so the mark-up isn't displayed.
+    incr word
+  } ;# while mark-up is $pagebreak or $turnaround
+  
+  if {[lindex $everything $word] == $space} {
+  #If a space mark-up is encountered
+    append output "$spacechar"
+    incr left -1
+  #Skip to the next word, so the mark-up isn't displayed.
+    incr word
+  } ;#while mark-up is $space
+ } ;#while there's mark-up  
+
   set lastcheck $check
   set check [lindex $everything $word]
 
@@ -625,7 +967,7 @@ global lastcheck
 	update
 	#puts  "\\n"
 	  for {set i 0} {$i < $left} {incr i} {
-		append output $spacechar
+		append output "$spacechar"
 	  }
 	  write_line $direction $output
 	  set output ""	
@@ -639,17 +981,24 @@ global lastcheck
 	  }
 	  incr word
 
+      if {$left < [expr $maxX * .1] } {
+        #if the cursor is too close to the margin,
+	#switch directions,
+	ps_empty
+      }
+	
+      if {[expr [string length [lindex $everything $word]] + 1] > $left} {
       #Check to see if the word is short enough to fit on the line.
-      if {[string length [lindex $everything $word]] > $left | \
-	$left < [expr $maxX * .1]} {
 	      #If not:
 	      	  #puts stderr "Too long first way"
 	     
 	     #If it's too long for the other way, it'll see which direction
 	     #provides the most room for the word.
 	     if {[whichfit $word] != 0 } { 
-	       set nonflipped [expr [string length [lindex $everything $word]] - $left]
-	       set flipped [expr [string length [lindex $everything $word]] - $maxX - $left -1]
+	       set nonflipped [expr [string length [lindex $everything $word]]\
+	       	   - $left]
+	       set flipped [expr [string length [lindex $everything $word]] \
+	       	   - $maxX - $left -1]
 	       set fits 0
 	     } else {
 	       #if it fits
@@ -666,7 +1015,7 @@ global lastcheck
 	        
 	        set direction [expr $direction * -1]
 		for {set i 0} {$i <= $left} {incr i} {
-			append output $spacechar
+			append output "$spacechar"
 	  	}
 		set left [expr $maxX - $left]
 		incr left -1	
@@ -682,7 +1031,7 @@ global lastcheck
 		if {$print} {
 		  #incr psline [expr $psfontsize * -1]
 		  set psline [expr $psline + [expr $psfontsize * -1.0] ]
-		  if {$psline <= $tbmargin } {
+		  if {$psline <= $tbm } {
 		    NewPage
 		  }
 		}
@@ -696,19 +1045,19 @@ global lastcheck
 	    #(Doesn't this just add buffer space to previous and next lines?
 	    # Leaving the word in the 'queue' for the next iteration to handle?)
 		for {set count 0} {$count < $maxX} {incr count} {
-		  append output2 $spacechar
+		  append output2 "$spacechar"
 		}	
 		write_line [expr $direction * -1] $output2
 		set output2 ""
 
 		for {set i 0} {$i < [expr $maxX - $left]} {incr i} {
-			append output $spacechar
+			append output "$spacechar"
 	  	}		
 		incr y
 		if {$print} {
 		  #incr psline [expr $psfontsize * -1]
 		  set psline [expr $psline + [expr $psfontsize * -1.0] ]
-		  if {$psline <= $tbmargin } {
+		  if {$psline <= $tbm } {
 		    NewPage
 		  }
 		}
@@ -729,7 +1078,7 @@ global lastcheck
 		incr x [expr [expr $size -1] * -1]
 	    }
 	    #
-	  append output "[lindex $everything $word] "
+	  append output "[lindex $everything $word]$spacechar"
 	  incr word
 	  incr left [expr $size * -1]
 	} else {
@@ -750,13 +1099,13 @@ global lastcheck
 	    
 	    if {$lastcheck != "\n"} {
 	      for {set i 0} {$i < $left} {incr i} {
-		append output $spacechar
+		append output "$spacechar"
 	      }
 	    }
 	    write_line $direction $output
 	    set output ""
 	    for {set i 0} {$i <= $left} {incr i} {
-		  append output $spacechar
+		  append output "$spacechar"
 	    }
 	    set left [expr $maxX - $left]
 	    incr left -1
@@ -765,7 +1114,7 @@ global lastcheck
 	    if {$print} {
 	      #incr psline [expr $psfontsize * -1]
 	      set psline [expr $psline + [expr $psfontsize * -1.0] ]
-	      if {$psline <= $tbmargin } {
+	      if {$psline <= $tbm } {
 	  	  NewPage
 	      }
 	    }
@@ -786,6 +1135,39 @@ if {$printOnly == 0} {
 #puts "end postformat"
 } ;# end of postformat
 
+proc ps_empty {} {
+ global direction
+ global left
+ global spacechar
+ global prnOut
+ global maxX
+ global print
+ global rongo
+
+	set direction [expr $direction * -1]
+	set left [expr $maxX - $left]
+	incr left -1
+	if {$print} {
+	  if { $direction > 0} {
+	    puts $prnOut "forward"
+          } else {
+            if { $rongo } {
+              puts $prnOut "upsidedown"
+            } else {
+              puts $prnOut "backward"
+            }
+          }
+	}
+
+     #Add empty line, but don't do 'nextline'
+	set bleh ""
+	for {set count 0} {$count < $maxX} {incr count} {
+	  append bleh "$spacechar"
+	}	
+	if {$print } {puts $prnOut "($bleh) show"}
+
+} ;# end ps_empty
+
 proc write_line {direction text} {
 global x
 global y
@@ -800,20 +1182,24 @@ global iscolor
 global print
 global printFile
 global prnOut
-
+global rongo
 
 	#regsub -all " \$" $text "" text
 	#regsub -all "\^ " $text "" text
  if { $print } {
 	regsub -all {\(} $text {\\(} text
 	regsub -all {\)} $text {\\)} text
-	#You need to make sure to espace-protect ()'s
+	#You need to make sure to escape-protect ()'s
 	#	as well as OTHER special characters of PostScript
 	#	like % or \, etc...
   if { $direction > 0} {
     puts $prnOut "forward\n($text) show\nnextline"
   } else {
-    puts $prnOut "backward\n($text) show\nnextline"
+    if { $rongo } {
+      puts $prnOut "upsidedown\n($text) show\nnextline"
+    } else {
+      puts $prnOut "backward\n($text) show\nnextline"
+    }
   }
  } else { 
 #display text instead
@@ -826,7 +1212,11 @@ global prnOut
   } else {
     set text [reverse "$text"]
     .read insert end "$text\n"	;#print a line here
-    .read tag add bw $y.0 $y.end	;#set_font backward
+    if {$rongo} {
+      .read tag add ud $y.0 $y.end	;#set_font upsidedown
+    } else {
+      .read tag add bw $y.0 $y.end	;#set_font backward
+    }
     if { $iscolor } {
     	.read tag add bwcolor $y.0 $y.end}
   }
@@ -880,6 +1270,8 @@ proc hyphenate {word} {
     set firsthalf [string range $theword 0 $cutoff ]
     if {[string index $firsthalf end] != "-"} {
       set firsthalf "$firsthalf-"
+      ## Leaving off the hyphen if there already is one on the end
+      ## might change the expected length and cause problems.
     }
 #    set secondhalf [string range $theword [expr $thesize/2 ] $thesize ]
     set secondhalf [string range $theword [expr $cutoff+1] $thesize]
@@ -903,10 +1295,13 @@ proc setpsfontsize {} {
  global maxX
  global userfontsize
  
+ set pw [expr $pagew * 72]
+ set lrm [expr $lrmargin * 72]
+ 
  #Note:  This formula -- however it works -- is for fixed-width fonts only.
  #       Perhaps only courier...
   if {$userfontsize == "unspecified"} {
-    set psfontsize [expr [expr double([expr $pagew - $lrmargin - $lrmargin])/ $maxX] * 1.666666667]
+    set psfontsize [expr [expr double([expr $pw - $lrm - $lrm])/ $maxX] * 1.666666667]
   } else {
     set psfontsize $userfontsize
   }
